package com.supwisdom.dlpay.framework.filter;


import com.supwisdom.dlpay.exception.ValidateCodeException;
import com.supwisdom.dlpay.framework.security.CodeUtil;
import com.supwisdom.dlpay.framework.security.MyAuthenticationFailureHandler;
import com.supwisdom.dlpay.framework.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;


@Component("validateCodeFilter")
public class ValidateCodeFilter extends OncePerRequestFilter {

  /**
   * 校验失败处理器
   */
  @Autowired
  private MyAuthenticationFailureHandler myAuthenticationFailureHandler;


  @Override
  protected void doFilterInternal(HttpServletRequest request,
                                  HttpServletResponse response, FilterChain filterChain)
      throws ServletException, IOException {
    String context = request.getContextPath();
    if (context == null || "" == context.trim()) {
      context = "/";
    }
    if (request.getRequestURI().isEmpty()) {
      filterChain.doFilter(request, response);
      return;
    }
    String url = request.getRequestURI();
    if (!"/".equals(context)) {
      url = url.replace(context, "");
    }
    if (StringUtil.equals("/login/form", url)
        && StringUtil.equalsIgnoreCase(request.getMethod(), "post")) {
      try {
        validate(request);
      } catch (ValidateCodeException e) {
        //response.setStatus(HttpStatus.OK.value());
        //response.setContentType("application/json;charset=UTF-8");
        //response.getWriter().write(objectMapper.writeValueAsString(JsonResult.error(400, e.getMessage())));
        //response.sendError(HttpStatus.UNAUTHORIZED.value(),e.getMessage());
        myAuthenticationFailureHandler.onAuthenticationFailure(request, response, e);
        return;
      }
    }
    filterChain.doFilter(request, response);
  }

  private void validate(HttpServletRequest request) throws ValidateCodeException {
    if (!CodeUtil.checkVerifyCode(request)) {
      throw new ValidateCodeException("验证码不匹配");
    } else {
      request.getSession().removeAttribute(com.google.code.kaptcha.Constants.KAPTCHA_SESSION_KEY);
    }
//    VerifyCode imageCode = (VerifyCode) request.getSession().getAttribute(ImageCodeUtil.LOGIN_IMAGECODE_SESSIONKEY);
//    String inputCode;
//    try {
//      inputCode = request.getParameter("imageCode");
//    } catch (Exception e) {
//      throw new ValidateCodeException("获取验证码的值失败");
//    }
//    if (StringUtil.isEmpty(inputCode)) {
//      throw new ValidateCodeException("验证码不能为空");
//    }
//    if (null == imageCode) {
//      throw new ValidateCodeException("验证码不存在");
//    }
//    if (imageCode.isExpired()) {
//      request.getSession().removeAttribute(ImageCodeUtil.LOGIN_IMAGECODE_SESSIONKEY);
//      throw new ValidateCodeException("验证码已过期");
//    }
//    if (!StringUtil.equalsIgnoreCase(imageCode.getText(), inputCode)) {
//      throw new ValidateCodeException("验证码不匹配");
//    }
//    request.getSession().removeAttribute(ImageCodeUtil.LOGIN_IMAGECODE_SESSIONKEY);
  }

}
